# *- coding: utf-8 -*-
"""
text colorisation functions  due to extendet use of the python3 print
function this is for python3 only
"""
from os import name as osname
from sys import stdout, stderr
__echo = stdout.write
__puke = stderr.write

# get color escape sequence from string
if osname == 'nt':
	# color faker function for windows compatibility
	def colortext(_, text): return text
else:
	def colortext(color, text):
		"""colortag prepending and end-tag appending function"""
		if osname == 'nt':
			return text
		string = '\033['
		__c = color
		if len(color) == 4:
			__c = color[1:]
			string = '%s01;'%(string)
		if __c == 'gre':
			string = '%s30m'%(string)
		if __c == 'red':
			string = '%s31m'%(string)
		if __c == 'grn':
			string = '%s32m'%(string)
		if __c == 'yel':
			string = '%s33m'%(string)
		if __c == 'blu':
			string = '%s34m'%(string)
		if __c == 'vio':
			string = '%s35m'%(string)
		if __c == 'cya':
			string = '%s36m'%(string)
		if __c == 'whi':
			string = '%s37m'%(string)
		__coloredtext = '%s%s\033[0m'%(string, text)
		return __coloredtext

# define 2 functions for each color
# one for normal and one for bold text
# blu, bblu, cya, bcya, gre, bgre, grn, bgrn, red, bred, vio, bvio, whi, bwhi, yel, byel

def blu(text):
	"""function for color blue"""
	return colortext('blu', text)
def bblu(text):
	"""function for color boldblue"""
	return colortext('bblu', text)

def cya(text):
	"""function for color cyan"""
	return colortext('cya', text)
def bcya(text):
	"""function for color boldcyan"""
	return colortext('bcya', text)

def gre(text):
	"""function for color grey"""
	return colortext('gre', text)
def bgre(text):
	"""function for color boldgrey"""
	return colortext('bgre', text)

def grn(text):
	"""function for color green"""
	return colortext('grn', text)
def bgrn(text):
	"""function for color boldgreen"""
	return colortext('bgrn', text)

def red(text):
	"""function for color red"""
	return colortext('red', text)
def bred(text):
	"""function for color boldred"""
	return colortext('bred', text)

def vio(text):
	"""function for color violet"""
	return colortext('vio', text)
def bvio(text):
	"""function for color boldviolet"""
	return colortext('bvio', text)

def whi(text):
	"""function for color white"""
	return colortext('whi', text)
def bwhi(text):
	"""function for color boldwhite"""
	return colortext('bwhi', text)

def yel(text):
	"""function for color guess what?  yellow ;)"""
	return colortext('yel', text)
def byel(text):
	"""function for color and you already guessed it... boldyellow"""
	return colortext('byel', text)

colors = {
    'blu': blu,
    'blue': blu,
    'bblu': blu,
    'boldblue': blu,
    'cya': cya,
    'cyan': cya,
    'bcya': bcya,
    'boldcyan': bcya,
    'gre': gre,
    'grey': gre,
    'bgre': bgre,
    'boldgrey': bgre,
    'grn': grn,
    'green': grn,
    'bgrn': bgrn,
    'boldgreen': bgrn,
    'red': red,
    'bred': bred,
    'boldred': bred,
    'vio': vio,
    'violet': vio,
    'vio': bvio,
    'violet': bvio,
    'whi': whi,
    'white': whi,
    'bwhi': bwhi,
    'boldwhite': bwhi,
    'yel': yel,
    'yellow': yel,
    'byel': byel,
    'boldyellow':byel
}

def colorize(item, color):
	if color in colors.keys():
		color = colors[color]
		item = color(item)
	elif item in colors.values():
		key = color(item)
	return item

# functions for some high frequent use cases:
def abort(*messages):
	"""
	prints all text in blu by using STDOUT but also kills its
	parent processes and returns 0 (OK) instead of 1 (ERROR)
	by default used for aborting on STRG+C (see message,
	for "KeyboardInterrupt" exceptions)
	"""
	if not messages:
		messages = ('\naborted by keystroke', )
	msgs = []
	for msg in messages:
		if (messages.index(msg) % 2) == 0:
			msgs.append(blu(msg))
		else:
			msgs.append(yel(msg))
	__echo('%s\n'%' '.join(msg for msg in msgs))
	exit(1)

def dialog(*args, **kwargs):
	if osname == 'nt':
		return input(' '.join(args))
	delim = ' '
	if 'sep' in kwargs.keys():
		delim = kwargs['sep']
	color = 'blu'
	if 'color' in kwargs.keys():
		color = kwargs['color']
	keycolor = 'yel'
	if 'keycolor' in kwargs.keys():
		keycolor = kwargs['keycolor']
	msgs = []
	for arg in args:
		if (args.index(arg) % 2) == 0:
			msgs.append(colortext(color, arg))
		else:
			msgs.append(colortext(keycolor, arg))
	return delim.join(msgs)

def anquest(*args, **kwargs):
	"""
	while i most often want to display error texts which heave
	one or more primary causes i want the text parts printed
	in red and the causes printed in yellow as follows
	"""
	args = list(args)
	prompt = False
	if 'prompt' in kwargs.keys():
		prompt = kwargs['prompt']
	delim = ' '
	if 'delim' in kwargs.keys():
		delim = kwargs['delim']
	if osname == 'nt':
		if not prompt:
			prompt = input
		return prompt(' '.join(args))
	yesno = False
	if 'yesno' in kwargs.keys():
		yesno = kwargs['yesno']
	pref = {'yes': ['y', ''], 'no': ['n']}
	if 'pref' in kwargs.keys():
		pref = kwargs['pref']
	y, n, pref = pref['yes'][0].upper(), pref['no'][0], pref['yes']
	ynq = '?'
	if not prompt:
		if 'color' not in kwargs.keys():
			kwargs['color'] = 'grn'
		# resetting >bold<
		ynq = '\033[0m%s'%colortext(kwargs['color'], '?')
		if yesno:
			ynq = '%s [%s/%s]'%(ynq, y, n)
		args[-1] = '%s%s\n'%(args[-1], ynq)
		__echo(dialog(*args, **kwargs))
		answer = input()
		if yesno:
			answer = True if answer.lower() in pref else False
	else:
		if args[-1][-1] != '?':
			args[-1] = '%s?'%args[-1]
		answer = prompt(delim.join(args))
	return answer

def error(*args, **kwargs):
	"""
	while i most often want to display error texts which heave
	one or more primary causes i want the text parts printed
	in red and the causes printed in yellow as follows
	"""
	if osname == 'nt':
		return __puke(' '.join(args))
	delim = ' '
	if 'sep' in kwargs.keys():
		delim = kwargs['sep']
	errfile = ''
	errline = ''
	end = '\n' if not 'end' in kwargs.keys() else kwargs['end']
	buzzword = 'ERROR:'
	if 'file' in kwargs.keys():
		errfile = '%s:'%(kwargs['file'])
	if 'line' in kwargs.keys():
		errline = '%s:'%(kwargs['line'])
	if 'warn' in kwargs.keys():
		buzzword = 'WARNING:'
	elif 'buzzword' in kwargs.keys():
		buzzword = kwargs['buzzword']
	if 'sep' in kwargs.keys():
		delim = kwargs['sep']
	msgs = [errfile+errline+red(buzzword)]
	for arg in args:
		if (args.index(arg) % 2) == 0:
			if not arg:
				continue
			msgs.append(red(arg))
		else:
			msgs.append(yel(arg))
	__puke('%s%s'%(str(delim).join(msg for msg in msgs), end))
	stderr.flush()

def fatal(*args, **kwargs):
	"""
	does exactly the same as "error" except it prints texts
	in bold and kills its parent processes
	"""
	if osname == 'nt':
		return __puke(''.join(args))
	delim = ' '
	if 'sep' in kwargs.keys():
		delim = kwargs['sep']
	errfile = ''
	errline = ''
	end = '\n' if not 'end' in kwargs.keys() else kwargs['end']
	if 'file' in kwargs.keys():
		errfile = '%s:'%(kwargs['file'])
	if 'line' in kwargs.keys():
		errline = '%s: '%(kwargs['line'])
	if 'sep' in kwargs.keys():
		delim = kwargs['sep']
	msgs = ['%s%s%s'%(errfile, errline, bred('FATAL:'))]
	for arg in args:
		if (args.index(arg) % 2) == 0:
			if not arg:
				continue
			msgs.append(bred(arg))
		else:
			msgs.append(yel(arg))
	__puke('%s%s'%(str(delim).join(msg for msg in msgs), end))
	stderr.flush()
	exit(1)

def tabs(dat, ind=0, ll=80):
	"""string indentations on newline"""
	return '\n'.join(
        '%s%s'%(' '*ind, dat[i:int(i+ll)]) for i in range(0, len(dat), ll))

def tabl(dats, ind=0, iind=0):
	"""list to string with indentations"""
	tabbl = ''
	for i in dats:
		if isinstance(i, (tuple, list)):
			tabbl = '%s\n%s'%(tabbl, tabl(
                i, ind+iind if iind else 2, iind if iind else 2))
			continue
		elif isinstance(i, dict):
			tabbl = '%s\n%s'%(tabbl, tabd(i, ind, iind))
			continue
		tabbl = '%s\n%s%s'%(tabbl, ' '*ind, i)
	return tabbl #.lstrip('\n')

def tabd(dats, ind=0, iind=0, acol=None, bcol=None):
	"""
	this is a function where i try to guess the best indentation for text
	representation of keyvalue paires with best matching indentation
	e.g:
	foo         = bar
	a           = b
	blibablubb  = bla
	^^indent "bar" and "b" as much as needed ("add" is added to each length)
	"""
	try:
		lim = int(max(len(str(k)) for k in dats if k)+int(ind))
	except (ValueError, TypeError):
		return dats
	tabbd = ''
	try:
		for (key, val) in sorted(dats.items()):
			ispc = ' '*ind if ind else ''
			spc = ' '*int(lim-len(str(key))-ind+1)
			if acol:
				key = colorize(key, acol)
			if val and isinstance(val, dict):
				tabbd = '%s\n%s%s:\n%s'%(tabbd, ispc, key, tabd(
                    val, ind+int(iind if iind else 2), iind if iind else 2))
			elif val and type(val) in (tuple, set, list):
				if bcol:
					val = colorize(val, bcol)
				tabbd = str('%s\n%s%s%s= %s'%(
                    tabbd, ispc, key, spc, val)).strip('\n')
			else:
				if bcol:
					val = colorize(val, bcol)
				tabbd = str('%s\n%s%s%s= %s'%(
                    tabbd, ispc, key, spc, val)).strip('\n')

	except AttributeError:
		return tabl(dats, ind)
	return tabbd.strip('\n')



"""x notification function"""
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
from os import environ

from system.user.find import userfind

def xenv():
	usr = userfind()
	home = userfind(usr, 'home')
	dmngr = '/org/freedesktop/DisplayManager'
	environ['XDG_RUNTIME_DIR'] = '/run/user/%s'%userfind(usr, 'uid')
	environ['XDG_SEAT_PATH'] = '%s/Seat0'%dmngr
	environ['XDG_SESSION_PATH'] = '%s/Session0'%dmngr
	environ['XDG_CONFIG_HOME'] = '%s/.config'%home
	environ['XAUTHORITY'] = '%s/.Xauthority'%home
	environ['DISPLAY'] = ':0'



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""passcrypt module"""

from sys import argv, stdout

from os import path, remove, environ, chmod, stat, makedirs

import socket

from time import time

from yaml import load, dump, FullLoader

from paramiko.ssh_exception import SSHException

from colortext import blu, yel, grn, bgre, tabd, error

from system import \
    userfind, filerotate, setfiletime, \
    xgetpass, xmsgok, xinput, xnotify, \
    filetime, absrelpath, xyesno, random, copy

from net.ssh import SecureSHell

from secrecy.gpgtools import GPGTool, GPGSMTool, DecryptError, SignatureError

from atexit import register

class PassCrypt(GPGTool, SecureSHell):
	"""passcrypt main class"""
	dbg = None
	vrb = None
	aal = None
	fsy = None
	sho = None
	rem = None
	rnd = None
	out = None
	gsm = None
	key = None
	sig = True
	gui = None
	chg = None
	syn = None
	try:
		user = userfind()
		home = userfind(user, 'home')
	except FileNotFoundError:
		user = environ['USERNAME']
		home = path.join(environ['HOMEDRIVE'], environ['HOMEPATH'])
	user = user if user else 'root'
	home = home if home else '/root'
	config = path.join(home, '.config', 'pwclip.cfg')
	plain = path.join(home, '.pwd.yaml')
	crypt = path.join(home, '.passcrypt')
	timefile = path.expanduser('~/.cache/PassCrypt.time')
	remote = ''
	reuser = user
	recvs = []
	key = ''
	keys = {}
	sslcrt = ''
	sslkey = ''
	sigerr = None
	genpwrex = None
	genpwlen = 24
	maxage = 3600
	__weaks = {}
	__oldweaks = {}
	def __init__(self, *args, **kwargs):
		"""passcrypt init function"""
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print(bgre(PassCrypt.__mro__))
			print(bgre(tabd(PassCrypt.__dict__, 2)))
			print(' ', bgre(self.__init__))
			print(bgre(tabd(self.__dict__, 4)))
		self.cpasmsg = '%s %s%s '%(
            blu('enter password for'), yel(self.crypt), blu(':'))
		gargs = list(args) + ['sig'] if self.sig else []
		if self.gui:
			gargs = list(args) + ['gui'] + ['sig'] if self.sig else []
			self.cpasmsg = 'enter password for %s: '%self.crypt
		if self.gsm or (
              self.gsm is None and self.recvs and [
                  r for r in self.recvs if r in GPGSMTool().keylist(True)]):
			GPGSMTool.__init__(self, *args, **kwargs)
		else:
			GPGTool.__init__(self, *args, **kwargs)
		self.keys = self.findkey()
		if not self.keys:
			self._mkconfkeys()
			SecureSHell.__init__(self, *args, **kwargs)
		if self._checktime():
			self._copynews()
		self.__weaks = self._readcrypt()
		self.__oldweaks = str(dict(sorted(self.__weaks.items())))
		self.__weaks = self._mergecrypt(self.__weaks)
		register(self._cryptpass)

	def __del__(self):
		self._cryptpass()

	def _cryptpass(self):
		chgs = []
		crecvs = []
		if path.isfile(self.crypt):
			erecvs = list(set(self.recvlist(self.crypt)))
			for r in erecvs:
				crecvs.append('0x%s'%r[-16:])
			for r in self.recvs:
				if r not in crecvs:
					chgs.append('+ %s'%r)
			for r in crecvs:
				if r not in self.recvs:
					chgs.append('- %s'%r)
		if self.__oldweaks != str(dict(sorted(self.__weaks.items()))) or chgs:
			msg = ('recipients have changed:\n',
                    '\n'.join(c for c in chgs),
                    '\nfrom:\n', ' '.join(erecvs),
                    '\nto:\n', ' '.join(self.recvs),
                    '\nencryption enforced...')
			if chgs:
				if not self.gui:
					error(*msg)
				else:
					xmsgok(' '.join(msg))
			if not self.recvs and not self.findkey():
				self.genkeys(self._gendefs(self.gui))
			if self.__weaks:
				if self._writecrypt(self.__weaks):
					try:
						self._copynews()
					except OSError:
						pass

	def _mkconfkeys(self):
		self.key = '0x%s'%str(self.genkeys())[-16:]
		cfgs = {'gpg': {}}
		override = False
		if path.isfile(self.config):
			override = True
			with open(self.config, 'r') as cfh:
				cfgs = dict(load(cfh.read(), Loader=Loader))
			if 'gpg' not in cfgs.keys():
				cfgs['gpg'] = {}
		cfgs['gpg']['key'] = self.key
		if 'recipients' not in cfgs['gpg'].keys():
			cfgs['gpg']['recipients'] = self.key
			self.recvs = [self.key]
		with open(self.config, 'w+') as cfh:
			cfh.write(str(dump(cfgs)))

	def _checktime(self):
		ok = True
		if self.maxage:
			tf = absrelpath(self.timefile)
			if not path.exists(path.dirname(tf)):
				makedirs(path.dirname(tf))
			now = int(time())
			if not path.exists(tf):
				with open(tf, 'w+') as tfh:
					tfh.write(str(now))
			with open(tf, 'r') as tfh:
				last = int(tfh.read())
			age = int(now-int(last))
			if age >= int(self.maxage):
				with open(tf, 'w+') as tfh:
					tfh.write(str(now))
				return True

	def _mergecrypt(self, __weaks):
		try:
			with open(self.plain, 'r') as pfh:
				__newweaks = load(pfh.read(), Loader=Loader)
			if not self.dbg:
				remove(self.plain)
		except FileNotFoundError:
			__newweaks = {}
		__newweaks = {} if not __newweaks else __newweaks
		for (su, ups) in __newweaks.items():
			for (usr, pwdcom) in ups.items():
				if su not in __weaks.keys():
					__weaks[su] = {}
				__weaks[su][usr] = pwdcom
		return __weaks

	def _copynews(self, force=None):
		"""copy new file method"""
		if self.dbg:
			print(bgre(self._copynews))
		if not self.rem:
			return
		if not self.remote:
			return
		if not self._checktime():
			return
		fs = (self.crypt,)
		if self.sig:
			fs = (self.crypt, '%s.sig'%self.crypt)
		isok = True
		for i in fs:
			ok = self.scpcompstats(
                i, path.basename(i),
                2, remote=self.remote, reuser=self.reuser)
			if ok:
				isok = True if ok and isok else False
				if self.gui:
					xnotify('file %s synced successfully'%path.basename(i))
				else:
					print(
                        blu('file'), yel(path.basename(i)),
                        blu('synced successfully'))
		return isok

	def _readcrypt(self):
		"""read crypt file method"""
		if self.dbg:
			print(bgre(self._readcrypt))
		__dct = {}
		try:
			__dct, err = self.decrypt(self.crypt)
		except DecryptError as err:
			error(err)
			exit(1)
		__dct = dict(load(str(__dct), Loader=FullLoader))
		if err:
			if err == 'SIGERR':
				if self.gui:
					yesno = xyesno('reencrypt, even though ' \
                        'the passcryt signature could not be verified?')
				else:
					print(grn('reencrypt, even though ' \
                        'the passcryt signature could not be verified?'),
                        '[Y/n]')
					yesno = input()
					yesno = True if yesno in ('', 'y') else False
				if yesno and __dct:
					self._writecrypt(__dct)
		return __dct

	def _writecrypt(self, __weaks):
		"""crypt file writing method"""
		if self.dbg:
			print(bgre(self._writecrypt))
		kwargs = {
            'output': self.crypt,
            'key': self.key,
            'recvs': self.recvs}
		isok = self.encrypt(message=dump(__weaks),  **kwargs)
		cfh = open(self.crypt, 'r')
		cfh.close()
		chmod(self.crypt, 0o600)
		now = int(time())
		setfiletime(self.crypt, (now, now))
		return isok

	def __askpwdcom(self, sysuser, usr, pwd, com, opw, ocom):
		if self.rnd:
			pwd = self.rndgetpass()
		if self.gui:
			if not pwd:
				pwd = xgetpass(
				    'as user %s: enter password for entry %s'%(sysuser, usr))
				pwd = pwd if pwd else opw
				if not pwd:
					xmsgok('password is needed if adding password')
					return
			if not com:
				com = xinput(
				    'enter comment (optional, "___" deletes the comment)')
			com = ocom if not com else com
			if com == '___':
				com = None
		else:
			if not pwd:
				print(blu('as user '), yel(sysuser), ': ', sep='')
				pwd = pwd if pwd else self.passwd(msg='%s%s%s%s: '%(
						blu('  enter '), yel('password '),
						blu('for entry '), yel('%s'%usr)))
				pwd = pwd if pwd else opw
				if not pwd:
					error('password is needed if adding password')
					return
			if not com:
				print(
                    blu('  enter '), yel('comment '),
                    blu('(optional, '), yel('___'),
                    blu(' deletes the comment)'), ': ', sep='', end='')
				com = input()
			com = ocom if not com else com
			if com == '___':
				com = None
		return [p for p in [pwd, com] if p is not None]

	def rndgetpass(self):
		while True:
			__pwd = random(self.genpwlen, self.genpwrex)
			yesno = False
			if self.gui:
				yesno = xyesno('use the following password: "%s"?'%__pwd)
			else:
				print('%s %s%s [Y/n]'%(
                    grn('use the following password:'),
                    yel(pwd), grn('?')), sep='')
				yesno = input()
				yesno = True if str(yesno).lower() in ('y', '') else False
			if yesno:
				break
		return __pwd

	def adpw(self, usr, pwd=None, com=None):
		"""password adding method"""
		if self.dbg:
			print(bgre(tabd({
                self.adpw: {'user': self.user, 'entry': usr,
                            'pwd': pwd, 'comment': com}})))
		if not self.aal:
			if self.user in self.__weaks.keys():
				if usr in self.__weaks[self.user]:
					if self.gui:
						xmsgok('entry %s already exists for user %s'%(
                            usr, self.user))
					else:
						error(
                            'entry', usr, 'already exists for user', self.user)
					return self.__weaks
				elif self.user not in self.__weaks.keys():
					self.__weaks[self.user] = {}
				try:
					__opw, __ocom = self.__weaks[self.user][usr]
				except (KeyError, ValueError):
					__opw, __ocom = None, None
				pwdcom = self.__askpwdcom(
                    self.user, usr, pwd, com, __opw, __ocom)
				if pwdcom:
					self.__weaks[self.user][usr] = [p for p in pwdcom if p]
		else:
			for u in self.__weaks.keys():
				if usr in self.__weaks[u].keys():
					if self.gui:
						xmsgok('entry %s already exists for user %s'%(usr, u))
					else:
						error('entry', usr, 'already exists for user', u)
					continue
				try:
					__opw, __ocom = self.__weaks[u][usr]
				except (KeyError, ValueError):
					__opw, __ocom = None, None
				pwdcom = self.__askpwdcom(
                    self.user, usr, pwd, com, __opw, __ocom)
				if pwdcom:
					self.__weaks[u][usr] = [p for p in pwdcom if p]
		return dict(self.__weaks)

	def chpw(self, usr, pwd=None, com=None):
		"""change existing password method"""
		if self.dbg:
			print(bgre(tabd({
                self.chpw: {'user': self.user, 'entry': usr, 'pwd': pwd}})))
		if not self.aal:
			if self.__weaks and self.user in self.__weaks.keys() and \
                  usr in self.__weaks[self.user].keys():
				try:
					__opw, __ocom = self.__weaks[self.user][usr]
				except (KeyError, ValueError):
					__opw, __ocom = None, None
				self.__weaks[self.user][usr] = self.__askpwdcom(
                    self.user, usr, pwd, com, __opw, __ocom)
			else:
				if self.gui:
					xmsgok('no entry named %s for user %s'%(usr, self.user))
				else:
					error('no entry named', usr, 'for user', self.user)
		else:
			for u in self.__weaks.keys():
				if usr not in self.__weaks[u].keys():
					if self.gui:
						xmsgok('entry %s does not exist for user %s'%(usr, u))
					else:
						error('entry', usr, 'does not exist for user', u)
					continue
				try:
					__opw, __ocom = self.__weaks[self.user][usr]
				except (KeyError, ValueError):
					__opw, __ocom = None, None
				self.__weaks[u][usr] = self.__askpwdcom(
                    self.user, usr, pwd, com, __opw, __ocom)
		return dict(self.__weaks)

	def rmpw(self, usr):
		"""remove password method"""
		if self.dbg:
			print(bgre(tabd({self.rmpw: {'user': self.user, 'entry': usr}})))
		if self.aal:
			__w = dict(self.__weaks)
			for u in __w.keys():
				try:
					del self.__weaks[u][usr]
					setattr(self, 'chg', True)
				except KeyError:
					if self.gui:
						xmsgok('entry %s not found as user %s'%(usr, u))
					else:
						error('entry', usr, 'not found as user', u)
				if not self.__weaks[u].keys():
					del self.__weaks[u]
		else:
			if self.user in self.__weaks.keys() and \
                  usr in self.__weaks[self.user].keys():
				del  self.__weaks[self.user][usr]
			else:
				if self.gui:
					xmsgok('entry %s not found as user %s'%(usr, self.user))
				else:
					error('entry', usr, 'not found as user', self.user)
			if self.user in self.__weaks.keys() \
                  and not self.__weaks[self.user].keys():
				del self.__weaks[self.user]
		return dict(self.__weaks)

	def lspw(self, usr=None, aal=None):
		"""password listing method"""
		if self.dbg and not self.gui:
			print(bgre(tabd({self.lspw: {'user': self.user, 'entry': usr}})))
		aal = True if aal else self.aal
		__ents = {}
		if self.__weaks:
			if aal:
				__ents = self.__weaks
				if usr:
					usrs = [self.user] + \
                        [u for u in self.__weaks.keys() if u != self.user]
					for user in usrs:
						if user in self.__weaks.keys() and \
                              usr in self.__weaks[user].keys():
							__ents = {usr: self.__weaks[user][usr]}
							break
			elif self.user in self.__weaks.keys():
				__ents = self.__weaks[self.user]
				if usr in __ents.keys():
					__ents = {usr: self.__weaks[self.user][usr]}
		return dict(__ents)

def lscrypt(usr, dbg=None):
	"""passlist wrapper function"""
	if dbg:
		print(bgre(lscrypt))
	__ents = {}
	if usr:
		__ents = PassCrypt().lspw(usr)
	return __ents




if __name__ == '__main__':
	exit(1)



"""path related functions"""
from re import search
from os import getcwd, chdir, chmod, walk, \
    readlink, utime, stat as osstat
from os.path import expanduser, islink, \
    isfile, isdir, abspath, join as pjoin, exists
from time import time
from shutil import copy2
from configparser import ConfigParser as _ConfPars
from json import load as _jsonload

from system.random import randin

def absrelpath(path, base=None):
	"""absrelpath for finding absolute paths in optional given base"""
	base = base if base else getcwd()
	path = path.strip("'")
	path = path.strip('"')
	if path.startswith('~'):
		path = expanduser(path)
	if islink(path):
		path = readlink(path)
	if '..' in path or not path.startswith('/'):
		pwd = getcwd()
		chdir(base)
		path = abspath(path)
		chdir(pwd)
	return path.rstrip('/')

def realpaths(pathlist, base=None):
	"""realpaths using realpath on multiple input files"""
	base = base if base else getcwd()
	paths = []
	for path in pathlist:
		if isinstance(path, (list, tuple)):
			#print('list/tuple')
			for _ in path:
				paths = [absrelpath(p, base) for p in path]
		elif isinstance(path, str):
			if ' ' in path:
				#print('liststring')
				paths = [absrelpath(p.strip(), base) for p in path.strip('[]').split(',')]
				break
			else:
				#print('string', path)
				paths.append(absrelpath(path, base))
	return paths

def confpaths(paths, conf, base=''):
	"""find configs in paths in base if file exists"""
	return list(set([pjoin(expanduser('~'), path[2:], conf) \
        for path in paths if path.startswith('~/') and \
        isfile(pjoin(expanduser('~'), path[2:], conf))] + \
        [pjoin(base, path[2:], conf) for path in \
        paths if path.startswith('./') and \
        isfile(pjoin(base, path[2:], conf))] + \
        [pjoin(base, path, conf) for path in paths if not \
        path.startswith('/') and not path.startswith('.') and \
        isfile(pjoin(base, path, conf))] + \
        [pjoin(path, conf) for path in paths if path.startswith('/') and \
        isfile(pjoin(path, conf))]))

def confdats(*confs):
	"""get ini data from config files"""
	cfg = _ConfPars()
	cfgdats = {}
	for conf in confs:
		cfg.read(conf)
		for section in cfg.sections():
			cfgdats[section] = dict(cfg[section])
	return cfgdats

def jconfdats(*confs):
	"""get json data from config files"""
	__dats = {}
	for conf in confs:
		with open(conf, 'r') as stream:
			for (key, val) in _jsonload(stream).items():
				__dats[key] = val
	return __dats

def unsorted(files):
	"""unsort given files"""
	unsorteds = []
	while len(unsorteds) != len(files):
		f = files[randin(len(files))]
		unsorteds.append(f)
	return unsorteds

def filetime(trg):
	"""local file-timestamp method"""
	return int(osstat(trg).st_mtime), int(osstat(trg).st_atime)

def setfiletime(trg, mtime=None, atime=None):
	"""local file-timestamp set method"""
	mt, at = filetime(trg)
	if mtime and not atime:
		atime = at
	elif atime and not mtime:
		mtime = mt
	utime(trg, (at, mt))

def filerotate(lfile, count=3, force=None):
	"""rotate given file by a maximum of count"""
	mode = osstat(lfile).st_mode
	mt, at = filetime(lfile)
	rtn = False
	for i in reversed(range(0, int(count))):
		old = '%s.%d'%(lfile, i)
		if i == 0:
			old = lfile
		new = '%s.%d'%(lfile, int(i+1))
		if not isfile(old):
			continue
		if oldf:
			mode = osstat(old).st_mode
			mt, at = filetime(old)
		elif newf:
			mode = osstat(new).st_mode
			mt, at = filetime(new)
		copy2(old, new)
		chmod(new, mode)
		setfiletime(new, mt, at)
		rtn = True
	return rtn

def filesiter(folder, random=False, includes=[], excludes=[]):
	"""iterate over files for given folder"""
	for (d, _, fs) in walk(absrelpath(folder)):
		if excludes and [i for i in excludes if search(i, d)]:
			continue
		if includes and not [i for i in includes if search(i, d)]:
			continue
		reordered = sorted if not random else unsorted
		fs = reordered(fs)
		for f in reordered(fs):
			if excludes and [i for i in excludes if search(i, f)]:
				continue
			if includes and not [i for i in includes if search(i, f)]:
				continue
			f = pjoin(d, f)
			yield f

def linesiter(target, includes=[], excludes=[]):
	__lns = []
	with open(target, 'r') as pfh:
		__lns = pfh.readlines()
	for l in __lns:
		if excludes and [i for i in excludes if search(i, l)]:
			continue
		if includes and not [i for i in includes if search(i, l)]:
			continue
		yield l

def findupper(path, name=None, dirs=None, files=None, links=None):
	"""find parent directory by given pattern"""
	if not name:
		name = path
		path = getcwd()
	while len(path.split('/')) > 1:
		trg = pjoin(path, name)
		if dirs is None and files is None and links is None and exists(trg):
			return trg
		elif links and islink(trg):
			return trg
		elif files and isfile(trg):
			return trg
		elif dirs and isdir(trg):
			return trg
		return findupper(
            '/'.join(p for p in path.split('/')[:-1]),
            name, dirs, files, links)

def findupperdir(path, name=None):
	"""find parent directory by given pattern"""
	return findupper(path, name, dirs=True)




from colortext import error

def dictreplace(srcdict, trgdict):
	if not isinstance(srcdict, dict):
		return error('type \'dict\' expected, got', type(trgdict))
	newdict = {}
	for (k, v) in srcdict.items():
		if k in trgdict.keys() and isinstance(trgdict[k], dict):
			__dict = dictreplace(srcdict[k], trgdict[k])
			if 'delkey' in trgdict[k].keys():
				for (ik, iv) in __dict.items():
					newdict[ik] = iv
			else:
				newdict[k] = __dict
			continue
		elif k in trgdict.keys():
			newdict[trgdict[k]] = v
		else:
			newdict[k] = v
	return newdict



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
"""pwclip main program"""
# global & stdlib imports
try:
	from os import fork
except ImportError:
	def fork(): """fork faker function""" ;return 0

from os import environ, path, remove, getpid, name as osname

from sys import exit

from argparse import ArgumentParser

from argcomplete import autocomplete
from argcomplete.completers import FilesCompleter, ChoicesCompleter

from socket import gethostname as hostname

from time import sleep

from yaml import load, FullLoader

from getpass import getpass

# local relative imports
from colortext import bgre, bred, tabd, error, fatal, abort

from executor import command as cmd

from system import \
    absrelpath, copy, paste, xgetpass, \
    xmsgok, xyesno, xnotify, xinput, which, whoami, dictreplace

from pwclip.passcrypt import PassCrypt, lscrypt

from secrecy import ykchalres, yubikeys

from pwclip.__pkginfo__ import version

def forkwaitclip(text, poclp, boclp, wait=3, out=None):
	"""clipboard forking, after time resetting function"""
	frk = fork()
	if frk == 0:
		if out == 'gui':
			cmd.call(str('xvkbd -no-keypad -delay 18 -text "%s"'%text).split())
		elif out == 'cli':
			print(text, end='')
		copy(text, mode='pb')
		try:
			sleep(int(wait))
		finally:
			copy(poclp, mode='p')
			copy(boclp, mode='b')
		exit(0)

def __passreplace(pwlist):
	"""returnes a string of asterisk's as long as the password is"""
	__pwcom = ['*'*len(str(pwlist[0]))]
	if len(pwlist) > 1:
		__pwcom.append(pwlist[1])
	return __pwcom

def __dctpwreplace(pwdict):
	"""password => asterisk replacement function"""
	__pwdict = {}
	for (usr, ent) in pwdict.items():
		if isinstance(ent, dict):
			__pwdict[usr] = {}
			for (u, e) in ent.items():
				__pwdict[usr][u] = __passreplace(e)
		elif ent:
			__pwdict[usr] = __passreplace(ent)
	return __pwdict

def _envconf(srcdict):
	newdict = {}
	for (k, v) in srcdict.items():
		if k in environ.keys():
			newdict[v] = environ[k]
	return newdict

def _printpws_(pwdict, insecure=False):
	"""password printer with in/secure option"""
	if not insecure:
		pwdict = __dctpwreplace(pwdict)
	print(tabd(pwdict))
	exit(0)

def optpars(cfgs, mode, name):
	desc = 'pwclip - Multi functional password manager to temporarily ' \
           'save passphrases to your copy/paste buffers for easy and ' \
           'secure accessing your passwords. The following ' \
           'arguments mights also be set by the config ' \
           '~/.config/%s.yaml file.'%name
	epic = 'the yubikey mode is compatible with the ' \
           'challenge-response feature of yubikeys only for now.'
	pars = ArgumentParser(description=desc, epilog=epic)
	pars.set_defaults(**cfgs)
	pars.add_argument(
        '--version',
        action='version', version='%(prog)s-v'+version)
	pars.add_argument(
        '-D', '--debug',
        dest='dbg', action='store_true', help='debugging mode')
	pars.add_argument(
        '-A', '--all',
        dest='aal', action='store_true',
        help='switch to all users entrys ("%s" only is default)'%cfgs['user'])
	pars.add_argument(
        '-o', '--stdout',
        dest='out', action='store_const', const=mode,
        help='print password to stdout (insecure and unrecommended)')
	pars.add_argument(
        '-g', '--genpw-pattern',
        dest='genpwrex', default=False, nargs='?',
        help='randomly generate password (only useful ' \
             'with -a, default is [\w#_;:\*\+\.\,\-\?\!]:24)')
	pars.add_argument(
        '-s', '--show-passwords',
        dest='sho', action='store_true',
        help='show passwords when listing (replaced by "*" is default)')
	pars.add_argument(
        '-t',
        dest='time', default=3, metavar='seconds', type=int,
        help='time to wait before resetting clip (%s is default)'%cfgs['time'])
	rpars = pars.add_argument_group('remote arguments')
	rpars.set_defaults(**cfgs)
	rpars.add_argument(
        '-R',
        dest='rem', action='store_true',
        help='use remote backup given by --remote-host/--remote-user')
	rpars.add_argument(
        '--remote-host',
        dest='remote', metavar='HOST',
        help='use HOST for connections')
	rpars.add_argument(
        '--remote-user',
        dest='reuser', metavar='USER',
        help='use USER for connections to HOST ("%s" is default)'%cfgs['user'])
	gpars = pars.add_argument_group('gpg/ssl arguments')
	gpars.set_defaults(**cfgs)
	gpars.add_argument(
        '-k', '--key',
        dest='key', metavar='ID', type=str,
        help='gpg-key ID(s) to use for decryption/signing')
	gpars.add_argument(
        '-r', '--recipients',
        dest='rvs', metavar='"ID [ID ...]"',
        help='one ore more gpg-key ID(s) to use for ' \
             'encryption (strings seperated by spaces within "")')
	gpars.add_argument(
        '-u', '--user',
        dest='usr', metavar='USER', default=cfgs['user'],
        help='query entrys only for USER (-A overrides, ' \
             '"%s" is default)'%cfgs['user'])
	gpars.add_argument(
        '-p', '--password',
        dest='pwd', default=None,
        help='enter password for add/change actions' \
             '(insecure & not recommended)')
	gpars.add_argument(
        '--comment',
        dest='com', default=None,
        help='enter comment for add/change actions')
	gpars.add_argument(
        '-x', '--x509',
        dest='gpv', action='store_const', const='gpgsm',
        help='force ssl compatible gpgsm mode - usually is autodetected ' \
             '(use --cert & --key for imports)')
	gpars.add_argument(
        '-C', '--cert',
        dest='sslcrt', metavar='SSL-Certificate',
        help='one-shot setting of SSL-Certificate')
	gpars.add_argument(
        '-K', '--ssl-key',
        dest='sslkey', metavar='SSL-Private-Key',
        help='one-shot setting of SSL-Private-Key')
	gpars.add_argument(
        '--ca', '--ca-cert',
        dest='sslca', metavar='SSL-CA-Certificate',
        help='one-shot setting of SSL-CA-Certificate')
	gpars.add_argument(
        '-P', '--passcrypt',
        dest='pcr', metavar='CRYPTFILE',
        default=path.expanduser('~/.passcrypt'),
        help='set location of CRYPTFILE to use as ' \
             'password store (~/.passcrypt is default)')
	gpars.add_argument(
        '-Y', '--yaml',
        dest='yml', metavar='YAMLFILE',
        default=path.expanduser('~/.pwd.yaml'),
        help='set location of YAMLFILE to read whole ' \
             'sets of passwords from a yaml file (~/.pwd.yaml is default)')
	ypars = pars.add_argument_group('yubikey arguments')
	ypars.set_defaults(**cfgs)
	ypars.add_argument(
        '-S', '--slot',
        dest='ysl', default=None, type=int, choices=(1, 2),
        help='set one of the two yubikey slots (only useful with -y)'
        ).completer = ChoicesCompleter((1, 2))
	ypars.add_argument(
        '-y', '--ykserial',
        nargs='?', dest='yks', metavar='SERIAL', default=False,
        help='switch to yubikey mode and optionally set ' \
		     'SERIAL of yubikey (autoselect serial and slot is default)')
	apars = pars.add_argument_group('action arguments')
	apars.set_defaults(**cfgs)
	apars.add_argument(
        '-a', '--add',
        dest='add', metavar='ENTRY', nargs='?' if mode == 'gui' else None,
        default=False, help='add ENTRY (password will be asked interactivly)')
	apars.add_argument(
        '-c', '--change',
        dest='chg', metavar='ENTRY', nargs='?' if mode == 'gui' else None,
        default=False, help='change ENTRY (password will be asked interactivly)')
	apars.add_argument(
        '-d', '--delete',
        dest='rms', metavar='ENTRY', nargs='?' if mode == 'gui' else '+',
        default=False, help='delete ENTRY(s) from the passcrypt list')
	apars.add_argument(
        '-l', '--list',
        nargs='?', dest='lst', metavar='PATTERN', default=False,
        help='pwclip an entry matching PATTERN if given ' \
             '- otherwise list all entrys')
	return pars

def confpars(mode):
	"""pwclip command line opt/arg parsing function"""
	_me = path.basename(path.dirname(__file__))
	cfg = path.expanduser('~/.config/%s.cfg'%_me)
	cfgs = {
        'crypt': path.expanduser('~/.passcrypt'),
        'plain': path.expanduser('~/.pwd.yaml'),
        'time': 3,
        'binary': 'gpg' if osname == 'nt' else 'gpg.exe',
        'user': whoami(),
        }
	try:
		with open(cfg, 'r') as cfh:
			confs = dict(load(cfh.read(), Loader=FullLoader))
	except (TypeError, FileNotFoundError):
		confs = {}
	cfgmap = {
        'gpg': {'recipients': 'rvs', 'delkey': True},
        'remote': {'user': 'reuser', 'host': 'remote', 'delkey': True},
        'yubikey': {'slot': 'ykslot', 'seerial': 'ykser', 'delkey': True}}
	envmap = {
        'GPGKEY': 'key',
        'RECIPIENTS': 'rvs',
        'PWCLIPTIME': 'time',
        'YKSERIAL': 'ykser',
        'USER': 'user',
        'USERNAME': 'user',
        'YKSLOT': 'ykslot'}
	confs = dictreplace(confs, cfgmap)
	for (k, v) in confs.items():
		cfgs[k] = v
	envs = _envconf(envmap)
	for (k, v) in envs.items():
		cfgs[k] = v
	cfgs['binary'] = 'gpg2'
	if osname == 'nt':
		cfgs['binary'] = 'gpg'
	if 'crypt' not in cfgs.keys():
		cfgs['crypt'] = path.expanduser('~/.passcrypt')
	elif 'crypt' in cfgs.keys() and cfgs['crypt'].startswith('~'):
		cfgs['crypt'] = path.expanduser(cfgs['crypt'])
	if 'plain' not in cfgs.keys():
		cfgs['plain'] = path.expanduser('~/.pwd.yaml')
	elif 'plain' in cfgs.keys() and cfgs['plain'].startswith('~'):
		cfgs['plain'] = path.expanduser(cfgs['plain'])
	pars = optpars(cfgs, mode, 'pwcli')
	autocomplete(pars)
	pars = optpars(cfgs, mode, 'pwclip')
	autocomplete(pars)
	args = pars.parse_args()
	pargs = [a for a in [
        'aal' if args.aal else None,
        'dbg' if args.dbg else None,
        'gsm' if args.gpv else None,
        'gui' if mode == 'gui' else None,
        'rem' if args.rem else None,
        'sho' if args.sho else None] if a]
	__bin = 'gpg2'
	if args.gpv:
		__bin = args.gpv
	if osname == 'nt':
		__bin = 'gpgsm.exe' if args.gpv else 'gpg.exe'
	pkwargs = {}
	pkwargs['binary'] = __bin
	pkwargs['sslcrt'] = args.sslcrt
	pkwargs['sslkey'] = args.sslkey
	pkwargs['timefile'] = path.expanduser('~/.cache/%s.time'%_me)
	if args.genpwrex or args.genpwrex is None:
		pargs.append('rnd')
		genpwrex = args.genpwrex
		if args.genpwrex is None:
			genpwrex = '[(?=.*\[A-z\])(?=.*\[A-Z\])' \
                       '(?=.*\[0-9\])(?=.*\[$@\])(?!.*\[iIoO\])]:24'
		genpwlen = 24
		if ':' in genpwrex:
			genpwrex, genpwlen = \
                ''.join(str(genpwrex).split(':')[:-1]), \
                str(genpwrex).split(':')[-1]
		pkwargs['genpwrex'] = genpwrex
		pkwargs['genpwlen'] = genpwlen
	if args.pcr:
		pkwargs['crypt'] = args.pcr
	if args.rvs:
		pkwargs['recvs'] = str(args.rvs).split(' ')
	if args.key:
		pkwargs['key'] = args.key
	if args.usr:
		pkwargs['user'] = args.usr
	if args.time:
		pkwargs['time'] = args.time
	if args.yml:
		pkwargs['plain'] = args.yml
	if args.rem:
		if hasattr(args, 'remote'):
			pkwargs['remote'] = args.remote
		if hasattr(args, 'reuser'):
			pkwargs['reuser'] = args.reuser
	if args.dbg:
		print(bgre(pars))
		print(bgre(tabd(args.__dict__, 2)))
		print(bgre('pargs:\n  %s\npkwargs:\n%s'%(pargs, tabd(pkwargs, 2))))
	if mode != 'gui' and (
          args.yks is False and args.lst is False and \
          args.add is False and args.chg is False and \
          args.rms is False and args.sslcrt is None and args.sslkey is None):
		pars.print_help()
		exit(0)
	if mode == 'gui':
		return args, pargs, pkwargs
	return args, pargs, pkwargs

def cli():
	args, pargs, pkwargs = confpars('cli')
	if not path.isfile(args.yml) and \
          not path.isfile(args.pcr) and args.yks is False:
		with open(args.yml, 'w+') as yfh:
			yfh.write("""---\n%s:  {}"""%args.usr)
	poclp, boclp = paste('pb')
	if args.yks or args.yks is None:
		if 'YKSERIAL' in environ.keys():
			ykser = environ['YKSERIAL']
		ykser = args.yks if args.yks else None
		if ykser and len(ykser) >= 6:
			ykser = ''.join(str(ykser)[-6:])
		res = ykchalres(getpass(), args.ysl, ykser)
		if not res:
			fatal('could not get valid response on slot ', args.ysl)
		forkwaitclip(res, poclp, boclp, args.time, args.out)
		exit(0)
	__ents = {}
	err = None
	if args.add:
		__ents = PassCrypt(*pargs, **pkwargs).adpw(
                           args.add, args.pwd, args.com)
		if not __ents:
			error('something went wrong while adding', args.add)
			exit(1)
		__pc = __ents[args.user][args.add]
		if __pc:
			if len(__pc) == 2 and osname != 'nt':
				xnotify('%s: %s'%(
                        args.lst, ' '.join(__pc[1:])), args.time)
			forkwaitclip(__pc[0], poclp, boclp, args.time, args.out)
	elif args.chg:
		if args.pwd:
			pkwargs['password'] = args.pwd
		__ents = PassCrypt(*pargs, **pkwargs).chpw(
            args.chg, args.pwd, args.com)
		if not args.aal:
			__ents[args.user]
		if not __ents:
			if [h for (u, es) in __ents.items() if args.chg in en.keys()]:
				exit(0)
			err = ('could not change entry', args.chg)
	elif args.rms:
		ers = []
		for r in args.rms:
			__ents = PassCrypt(*pargs, **pkwargs).rmpw(r)
			if not args.aal:
				__ents[args.user]
			if r in __ents.keys():
				ers.append(r)
		ewrd = 'entry'
		if len(ers) >= 1:
			ewrd = 'entrys'
		err = ('deleting the following %s failed:', bred(', ').join(
               ers)) if ers else None
	elif args.lst is not False and args.lst is not None:
		__ents = PassCrypt(*pargs, **pkwargs).lspw(args.lst)
		if __ents and args.lst not in __ents.keys():
			err = (
                'could not find entry', args.lst,
                'for', args.user, 'in', pkwargs['crypt'])
		elif args.lst and __ents:
			__pc = __ents[args.lst]
			if __pc:
				if len(__pc) == 2 and osname != 'nt':
					xnotify('%s: %s'%(
                        args.lst, ' '.join(__pc[1:])), args.time)
				forkwaitclip(__pc[0], poclp, boclp, args.time, args.out)
				exit(0)
	elif args.lst is None:
		__ents = PassCrypt(*pargs, **pkwargs).lspw()
		err = 'no password entrys or decryption failed' if not __ents else None
	if err:
		fatal(err)
	_printpws_(__ents, args.sho)

def __xdialog(msg, sec=None):
	while True:
		rtn = False
		if sec:
			rtn = xgetpass(msg)
		else:
			rtn = xinput(msg)
		if not rtn:
			yesno = xyesno('no input received, abort?')
			if yesno in (True, None):
				rtn = False
				break
		if rtn:
			break
	return rtn

def gui(typ='pw'):
	"""gui wrapper function to not run unnecessary code"""
	poclp, boclp = paste('pb')
	args, pargs, pkwargs = confpars('gui')
	if args.yks or args.yks is None or typ == 'yk':
		res = ykchalres(xgetpass(), args.ykslot, args.ykser)
		if not res:
			xmsgok('no response from the key (if there is one)'%__in)
			exit(1)
		forkwaitclip(res, poclp, boclp, args.time, args.out)
	__ents = None
	if args.add is not False:
		_add = __xdialog(
            'as %s: enter name for the new password entry'%args.user)
		if not _add:
			xnotify('cannot add empty string')
			exit(1)
		if _add:
			__ents = PassCrypt(*pargs, **pkwargs).adpw(_add, None, None)
			if not __ents or args.user not in __ents or \
                  _add not in __ents[args.user].keys():
				xmsgok('something went wrong while adding %s'%_add)
				exit(1)
			__pc = __ents[args.user][_add]
			if len(__pc) == 2:
				xnotify('%s: %s'%(_add, ' '.join(__pc[1:])))
			forkwaitclip(__pc[0], poclp, boclp, args.time, args.out)
			umsg = 'all users'
			if not args.aal:
				umsg = 'user %s'%args.user
			xnotify('added entry %s for %s'%(_add, umsg))
	elif args.chg is not False:
		_chg = __xdialog(
            'as %s: enter name of the password entry to change'%args.user)
		if not _chg:
			xnotify('cannot change empty string')
			exit(1)
		if _chg:
			__ents = PassCrypt(*pargs, **pkwargs).chpw(_chg, None, None)
			if not __ents or args.user not in __ents or \
                  _chg not in __ents[args.user].keys():
				xmsgok('something went wrong while changing %s'%_chg)
				exit(1)
			__pc = __ents[args.user][_chg]
			if len(__pc) == 2:
				xnotify('%s: %s'%(_chg, ' '.join(__pc[1:])), args.time)
			forkwaitclip(__pc[0], poclp, boclp, args.time, args.out)
			xnotify('changed entry %s for %s'%(_chg, args.user))
	elif args.rms is not False:
		_rms = __xdialog(
            'as %s: enter name of the password entry(s) to delete'%args.user)
		if not _rms:
			xnotify('cannot not delete emty string')
			exit(1)
		if _rms and ' ' in _rms:
			_rms = [r.strip() for r in _rms.split(' ')]
		else:
			_rms = [_rms]
		for r in _rms:
			pcr = PassCrypt(*pargs, **pkwargs)
			__ents = pcr.rmpw(r)
			pcr.__del__()
			if not __ents:
				xnotify('could not delete entry %s'%r)
			else:
				xnotify('deleted entry %s for %s'%(r, args.user))
	elif args.lst is not False:
		_umsg = '%s\'s entrys'%args.usr
		if args.aal:
			_umsg = 'all entrys'
		__in = __xdialog('enter name to search in %s'%_umsg, True)
		if not __in:
			exit(1)
		__ent = PassCrypt(*pargs, **pkwargs).lspw(__in)
		if not __ent or (__ent and __in not in __ent.keys() or not __ent[__in]):
			xnotify('either %s or %s where not found in passcrypt'%(
                args.user, __in))
			exit(1)
		if __ent:
			__pc = __ent[__in]
			if __pc:
				if len(__pc) == 2:
					xnotify(' '.join(__pc[1:]))
				forkwaitclip(__pc[0], poclp, boclp, args.time, args.out)
	else:
		__ents = PassCrypt(*pargs, **pkwargs).lspw()
	if __ents:
		if args.aal:
			pargs.append('aal')
		if not args.sho:
			__ents = __dctpwreplace(__ents)
		xnotify(tabd(__ents))



#!/usr/bin/env python3
"""ssh connection and remote command """

#global imports"""
import os
from os.path import basename, isfile
import sys
import stat
from shutil import copy2, copyfile
from socket import \
    gaierror as NameResolveError, timeout as sockettimeout
from psutil import Process

from paramiko import AutoAddPolicy, SSHException, ssh_exception
from paramiko.client import SSHClient
from paramiko.agent import AgentRequestHandler

from colortext import bgre, tabd, abort, error
from executor import Command
from system import which, whoami, userfind, filetime, setfiletime, filerotate
from net import askdns

# default vars
__version__ = '0.1'

class SecureSHell(Command):
	"""paramiko wrapper class"""
	dbg = None
	reuser = whoami()
	remote = ''
	ssh = None
	def __init__(self, *args, **kwargs):
		"""ssh init function"""
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print(bgre(SecureSHell.__mro__))
			print(bgre(tabd(self.__dict__, 2)))
		Command.__init__(self, *args, **kwargs)

	def _ssh(self, remote=None, reuser=None, port=22):
		"""ssh connector method"""
		remote = remote if remote else self.remote
		reuser = reuser if reuser else self.reuser
		if '@' in remote:
			reuser, remote = remote.split('@')
		reuser = whoami() if not reuser else reuser
		if self.dbg:
			print(bgre('%s\n  %s@%s:%s '%(
                self._ssh, reuser, remote, port)))
		__ssh = SSHClient()
		__ssh.set_missing_host_key_policy(AutoAddPolicy())
		try:
			__ssh.connect(
                askdns(remote), int(port),
                username=reuser, allow_agent=True, look_for_keys=True)
		except TypeError as err:
			return error(err)
		ses = __ssh.get_transport().open_session()
		agent = AgentRequestHandler(ses)
		self.ssh = __ssh
		return self.ssh

	def rrun(self, cmd, remote=None, reuser=None):
		"""remote run method"""
		if self.dbg:
			print(bgre('%s\n  cmd = %s'%(self.rrun, cmd)))
		ssh = self.ssh if self.ssh else self._ssh(remote, reuser)
		try:
			ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rrun, err)

	def rcall(self, cmd, remote=None, reuser=None):
		"""remote call method"""
		if self.dbg:
			print(bgre('%s\n  cmd = %s'%(self.rcall, cmd)))
		ssh = self.ssh if self.ssh else self._ssh(remote, reuser)
		try:
			chn = ssh.get_transport().open_session()
			chn.settimeout(10800)
			chn.exec_command(cmd)
			while not chn.exit_status_ready():
				if chn.recv_ready():
					och = chn.recv(1024)
					while och:
						sys.stdout.write(och.decode())
						och = chn.recv(1024)
				if chn.recv_stderr_ready():
					ech = chn.recv_stderr(1024)
					while ech:
						sys.stderr.write(ech.decode())
						ech = chn.recv_stderr(1024)
			return int(chn.recv_exit_status())
		except (
               AttributeError, ssh_exception.SSHException, sockettimeout
               ) as err:
			error(self.rcall, err)
			raise err
		except KeyboardInterrupt:
			abort()

	def rstdx(self, cmd, remote=None, reuser=None):
		"""remote stout/error method"""
		if self.dbg:
			print(bgre('%s\n  cmd = %s'%(self.rstdx, cmd)))
		ssh = self.ssh if self.ssh else self._ssh(remote, reuser)
		try:
			_, out, err = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rstdx, err)
		return ''.join(out.readlines()), ''.join(err.readlines())

	def rstdo(self, cmd, remote=None, reuser=None):
		"""remote stdout method"""
		if self.dbg:
			print(bgre('%s\n  cmd = %s'%(self.rstdo, cmd)))
		ssh = self.ssh if self.ssh else self._ssh(remote, reuser)
		try:
			_, out, _ = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			return error(self.rstdo, err)
		return ''.join(out.readlines())

	def rstde(self, cmd, remote=None, reuser=None):
		"""remote stderr method"""
		if self.dbg:
			print(bgre('%s\n  cmd = %s'%(self.rstde, cmd)))
		ssh = self.ssh if self.ssh else self._ssh(remote, reuser)
		ssh = self._ssh(remote, reuser)
		try:
			_, _, err = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rstde, err)
		return ''.join(err.readlines())

	def rerno(self, cmd, remote=None, reuser=None):
		"""remote error code  method"""
		if self.dbg:
			print(bgre('%s\n  cmd = %s'%(self.rerno, cmd)))
		ssh = self.ssh if self.ssh else self._ssh(remote, reuser)
		ssh = self._ssh(remote, reuser)
		try:
			_, out, _ = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rerno, err)
		return int(out.channel.recv_exit_status())

	def roerc(self, cmd, remote=None, reuser=None):
		"""remote stdout/stderr/errorcode method"""
		if self.dbg:
			print(bgre('%s\n  cmd = %s'%(self.roerc, cmd)))
		ssh = self.ssh if self.ssh else self._ssh(remote, reuser)
		ssh = self._ssh(remote, reuser)
		try:
			_, out, err = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.roerc, err)
		return ''.join(out.readlines()), ''.join(err.readlines()), \
            out.channel.recv_exit_status()

	def get(self, src, trg, remote=None, reuser=None):
		"""sftp get method"""
		if not (os.path.isfile(src) or os.path.isfile(trg)):
			raise FileNotFoundError('connot find either %s nor %s'%(src, trg))
		remote = remote if remote else self.remote
		reuser = reuser if reuser else self.reuser
		cmd = '%s %s@%s:%s %s'%(which('scp'), reuser, remote, src, trg)
		if self.dbg:
			print(bgre('%s\n  src = %s\n  trg = %s\n  cmd = %s\n'%(
                self.get, src, trg, cmd)))
		eno = self.erno(cmd)
		return True if eno == 0 else False

	def put(self, src, trg, remote=None, reuser=None):
		"""sftp put method"""
		if not (os.path.isfile(src) or os.path.isfile(trg)):
			raise FileNotFoundError('connot find either %s nor %s'%(src, trg))
		remote = remote if remote else self.remote
		reuser = reuser if reuser else self.reuser
		cmd = '%s %s %s@%s:%s'%(
            which('scp'), src, reuser, remote, basename(trg))
		if self.dbg:
			print(bgre('%s\n  src = %s\n  trg = %s\n  cmd = %s\n'%(
                self.put, src, trg, cmd)))
		eno = self.erno(cmd)
		return True if eno == 0 else False

	def rcompstats(self, src, trg, remote=None, reuser=None):
		"""remote file-stats compare """
		smt = int(os.stat(src).st_mtime)
		if self.dbg:
			print(bgre('%s\n  src = %s\n  trg = %s'%(self.rcompstats, src, trg)))
		rmt = self.rstdo(
            'stat -c %%Y %s'%trg, remote=remote, reuser=reuser)
		if rmt:
			rmt = int(str(rmt))
		srctrg = src, '%s@%s:%s'%(reuser, remote, trg)
		if rmt == smt:
			return
		elif rmt and int(rmt) > int(smt):
			srctrg = '%s@%s:%s'%(reuser, remote, trg), src
		return srctrg

	def rfiletime(self, trg, remote=None, reuser=None):
		"""remote file-timestamp method"""
		if self.dbg:
			print(bgre('%s\n trg = %s'%(self.rfiletime, trg)))
		tamt = str(self.rstdo(
            'stat -c "%%X %%Y" %s'%trg, remote, reuser).strip())
		tat = 0
		tmt = 0
		if tamt:
			tat, tmt = tamt.split(' ')
		return int(tmt), int(tat)

	def rsetfiletime(self, trg, mtime, atime, remote=None, reuser=None):
		"""remote file-timestamp set method"""
		if self.dbg:
			print(bgre('%s\n trg = %s\n  mtime = %s\n  atime = %d'%(
			    self.rsetfiletime, trg, mtime, atime)))
		self.rstdo(
            'touch -m --date=@%s %s'%(mtime, trg), remote, reuser)
		self.rstdo(
            'touch -a --date=@%s %s'%(atime, trg), remote, reuser)

	def scpcompstats(self, lfile, rfile, rotate=0, remote=None, reuser=None):
		"""
		remote/local file compare method copying and
		setting the file/timestamp of the neweer one
		"""
		if self.dbg:
			print(bgre(self.scpcompstats))
			print(bgre('  %s@%s:%s %s'%(reuser, remote, rfile, lfile)))
		if not isfile(lfile):
			return
		lmt, lat = filetime(lfile)
		rmt, rat = self.rfiletime(rfile, remote, reuser)
		ok = False
		if rmt == lmt:
			ok = True
		if rmt and rmt > lmt:
			if rotate > 0:
				filerotate(lfile, rotate)
			eno = self.get(rfile, lfile, remote, reuser)
			ok = True if eno == 0 else False
			omode = int(self.rstdo('stat -c%%a %s'%rfile, remote, reuser), 8)
			os.chmod(lfile, omode)
			setfiletime(lfile, rmt, rat)
		elif lmt and lmt > rmt:
			omode = str(oct(stat.S_IMODE(os.stat(lfile).st_mode)))
			eno = self.put(lfile, rfile, remote, reuser)
			ok = True if eno == 0 else False
			self.rsetfiletime(rfile, lmt, lat, remote, reuser)
			self.rcall('chmod %s%s %s'%(
                omode[:1], omode[-3:], rfile))
			if rotate > 0:
				filerotate(lfile, rotate)
		return ok



if __name__ == '__main__':
	"""module debugging area"""
	#ssh = SecureSHell(**{'remote':'bigbox.janeiskla.de'})
	#print(ssh.command('cat /etc/debian_version'))



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
"""
clips - clipboard for various systems
"""
from os import environ, name as osname

from platform import system

from time import sleep, time

from executor import command as cmd

from subprocess import Popen, PIPE, DEVNULL

def clips():
	"""return `copy`, `paste` as system independent functions"""
	def winclips():
		global HGLOBAL, LPVOID, DWORD, LPCSTR, \
			   INT, HWND, HINSTANCE, HMENU, BOOL, UINT, HANDLE
		import contextlib
		from ctypes import \
			c_size_t, sizeof, c_wchar_p, \
			get_errno, c_wchar, windll, CDLL, memmove
		from ctypes.wintypes import \
			HGLOBAL, LPVOID, DWORD, \
			LPCSTR, INT, HWND, HINSTANCE, HMENU, BOOL, UINT, HANDLE
		class CheckedCall(object):
			def __init__(self, f):
				super(CheckedCall, self).__setattr__("f", f)
			def __call__(self, *args):
				ret = self.f(*args)
				if not ret and get_errno():
					raise RuntimeError("Error calling " + self.f.__name__)
				return ret
			def __setattr__(self, key, value):
				setattr(self.f, key, value)
		msvcrt = CDLL('msvcrt')
		mkwin = CheckedCall(windll.user32.CreateWindowExA)
		mkwin.argtypes = [
                          DWORD, LPCSTR, LPCSTR,
                          DWORD, INT, INT, INT, INT,
                          HWND, HMENU, HINSTANCE, LPVOID]
		mkwin.restype = HWND
		rmwin = CheckedCall(windll.user32.DestroyWindow)
		rmwin.argtypes = [HWND]
		rmwin.restype = BOOL
		mkclip = windll.user32.OpenClipboard
		mkclip.argtypes = [HWND]
		mkclip.restype = BOOL
		doclip = CheckedCall(windll.user32.CloseClipboard)
		doclip.argtypes = []
		doclip.restype = BOOL
		noclip = CheckedCall(windll.user32.EmptyClipboard)
		noclip.argtypes = []
		noclip.restype = BOOL
		isclip = CheckedCall(windll.user32.GetClipboardData)
		isclip.argtypes = [UINT]
		isclip.restype = HANDLE
		onclip = CheckedCall(windll.user32.SetClipboardData)
		onclip.argtypes = [UINT, HANDLE]
		onclip.restype = HANDLE
		alloc = CheckedCall(windll.kernel32.GlobalAlloc)
		alloc.argtypes = [UINT, c_size_t]
		alloc.restype = HGLOBAL
		dolock = CheckedCall(windll.kernel32.GlobalLock)
		dolock.argtypes = [HGLOBAL]
		dolock.restype = LPVOID
		unlock = CheckedCall(windll.kernel32.GlobalUnlock)
		unlock.argtypes = [HGLOBAL]
		unlock.restype = BOOL
		wcslen = CheckedCall(msvcrt.wcslen)
		wcslen.argtypes = [c_wchar_p]
		wcslen.restype = UINT
		GMEM_MOVEABLE = 0x0002
		CF_UNICODETEXT = 13
		@contextlib.contextmanager
		def window():
			hwnd = mkwin(
                0, b"STATIC", None, 0, 0, 0, 0, 0,
                None, None, None, None)
			try:
				yield hwnd
			finally:
				rmwin(hwnd)
		@contextlib.contextmanager
		def clipboard(hwnd):
			t = time() + 0.5
			success = False
			while time() < t:
				success = mkclip(hwnd)
				if success:
					break
				time.sleep(0.01)
			if not success:
				raise PyperclipWindowsException("Error calling mkclip")
			try:
				yield
			finally:
				doclip()
		def _copy(text, mode='pb'):
			with window() as hwnd:
				with clipboard(hwnd):
					noclip()
					if text:
						count = wcslen(text) + 1
						handle = alloc(
                            GMEM_MOVEABLE,
                            count * sizeof(c_wchar))
						locked_handle = dolock(handle)
						memmove(
                            c_wchar_p(locked_handle),
                            c_wchar_p(text), count * sizeof(c_wchar))
						unlock(handle)
						onclip(CF_UNICODETEXT, handle)
		def _paste(_=None):
			with clipboard(None):
				handle = isclip(CF_UNICODETEXT)
				if not handle:
					__clp = ""
				__clp = c_wchar_p(handle).value, c_wchar_p(handle).value
			return __clp
		return _copy, _paste

	def osxclips():
		""""OSX clipboards"""
		def _copy(text, mode=None):
			"""osx copy function"""
			text = text if text else ''
			if mode != 'b':
				with Popen(['pbcopy'], stdin=PIPE, close_fds=True) as prc:
					prc.communicate(input=str(text).encode('utf-8'))
			return False
		def _paste(_=None):
			"""osx paste function"""
			with Popen(['pbpaste'], stdout=PIPE, close_fds=True) as prc:
				out, _ = prc.communicate()
				return out.decode('utf-8'), None
		
		return _copy, _paste

	def linclips():
		"""linux clipboards"""
		dsp = ':0'
		if 'DISPLAY' in environ.keys():
			dsp = '%s'%(environ['DISPLAY'])
		environ['DISPLAY'] = dsp
		def _copy(text, mode='p'):
			"""linux copy function"""
			xsel = 'xsel -l /dev/null -i'
			text = text if text else ''
			for m in mode:
				xsel = '%s -%s'%(xsel, m)
				with Popen(
                      xsel, stdin=PIPE, stderr=DEVNULL,
                      stdout=DEVNULL, shell=True) as prc:
					prc.communicate(input=str(text).encode('utf-8'))
					prc.terminate()
		def _paste(mode='p'):
			"""linux paste function"""
			if mode == 'p':
				with Popen(
                      'xsel -l /dev/null -n -p',
                      stdout=PIPE, stderr=DEVNULL, shell=True) as prc:
					out, _ = prc.communicate()
				ret = out.decode()
			elif mode == 'b':
				with Popen(
                      'xsel -l /dev/null -n -b',
                      stdout=PIPE, stderr=DEVNULL, shell=True) as prc:
					out, _ = prc.communicate()
				ret = out.decode()
			else:
				with Popen(
                      'xsel -l /dev/null -n -p',
                      stdout=PIPE, stderr=DEVNULL, shell=True) as prc:
					pout, _ = prc.communicate()
				with Popen(
                      'xsel -l /dev/null -n -b',
                      stdout=PIPE, stderr=DEVNULL, shell=True) as prc:
					bout, _ = prc.communicate()
				ret = pout.decode(), bout.decode()
			return ret
		return _copy, _paste
	# decide which copy, paste functions to return [windows|mac|linux] mainly
	if osname == 'nt' or system() == 'Windows':
		return winclips()
	elif osname == 'mac' or system() == 'Darwin':
		return osxclips()
	return linclips()

copy, paste = clips()
